@@ -452,7 +452,9 @@ static void check_open_close_receive(DVCMAN_CHANNEL* channel)
 	WINPR_ASSERT(cb);
 	if (!cb->OnOpen || !cb->OnClose || !cb->OnDataReceived)
 		WLog_VRB(TAG, "{%s:%" PRIu32 "} OnOpen=%p, OnClose=%p, OnDataReceived=%p", name, id,
-		         cb->OnOpen, cb->OnClose, cb->OnDataReceived);
+		         WINPR_CXX_COMPAT_CAST(const void*, cb->OnOpen),
+		         WINPR_CXX_COMPAT_CAST(const void*, cb->OnClose),
+		         WINPR_CXX_COMPAT_CAST(const void*, cb->OnDataReceived));
 }
 
 static UINT dvcman_call_on_receive(DVCMAN_CHANNEL* channel, wStream* data)
@@ -507,16 +509,16 @@ static UINT dvcman_channel_close(DVCMAN_CHANNEL* channel, BOOL perRequest, BOOL
 			channel->state = DVC_CHANNEL_CLOSED;
 
 			{
+				check_open_close_receive(channel);
+
 				IWTSVirtualChannelCallback* cb = channel->channel_callback;
+				channel->channel_callback = NULL;
 				if (cb)
 				{
-					check_open_close_receive(channel);
 					IFCALL(cb->OnClose, cb);
 				}
 			}
 
-			channel->channel_callback = NULL;
-
 			if (channel->dvcman && channel->dvcman->drdynvc)
 			{
 				if (context)
@@ -794,14 +796,13 @@ out:
  */
 static UINT dvcman_open_channel(drdynvcPlugin* drdynvc, DVCMAN_CHANNEL* channel)
 {
-	IWTSVirtualChannelCallback* pCallback = NULL;
 	UINT error = CHANNEL_RC_OK;
 
 	WINPR_ASSERT(drdynvc);
 	WINPR_ASSERT(channel);
 	if (channel->state == DVC_CHANNEL_RUNNING)
 	{
-		pCallback = channel->channel_callback;
+		IWTSVirtualChannelCallback* pCallback = channel->channel_callback;
 
 		if (pCallback->OnOpen)
 		{
