@@ -121,6 +121,8 @@ typedef struct
 #endif
 
 	/* sws_scale */
+	uint32_t swsWidth;
+	uint32_t swsHeight;
 	struct SwsContext* sws;
 
 } CameraDeviceStream;
@@ -265,6 +267,7 @@ UINT ecam_channel_write(CameraPlugin* ecam, GENERIC_CHANNEL_CALLBACK* hchannel,
 void ecam_dev_destroy(CameraDevice* dev);
 
 WINPR_ATTR_MALLOC(ecam_dev_destroy, 1)
+WINPR_ATTR_NODISCARD
 CameraDevice* ecam_dev_create(CameraPlugin* ecam, const char* deviceId, const char* deviceName);
 
 /* video encoding interface */
