@@ -133,7 +133,7 @@ static DWORD WINAPI smartcard_context_thread(LPVOID arg)
 				{
 					element->irp->Discard(element->irp);
 					smartcard_operation_free(&element->operation, TRUE);
-					WLog_ERR(TAG, "smartcard_irp_device_control_call failed with error %" PRIu32 "",
+					WLog_ERR(TAG, "smartcard_irp_device_control_call failed with error %" PRId32 "",
 					         status);
 					break;
 				}
@@ -227,7 +227,6 @@ void smartcard_context_free(void* pCtx)
 	/* cancel blocking calls like SCardGetStatusChange */
 	WINPR_ASSERT(pContext->smartcard);
 	smartcard_call_cancel_context(pContext->smartcard->callctx, pContext->hContext);
-	smartcard_call_context_signal_stop(pContext->smartcard->callctx, FALSE);
 
 	if (pContext->IrpQueue)
 	{
@@ -714,7 +713,8 @@ FREERDP_ENTRY_POINT(UINT VCAPITYPE DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POIN
 
 	if (pEntryPoints->device->Name)
 	{
-		smartcard_call_context_add(smartcard->callctx, pEntryPoints->device->Name);
+		if (!smartcard_call_context_add(smartcard->callctx, pEntryPoints->device->Name))
+			goto fail;
 	}
 
 	sSmartcard = smartcard;
