@@ -36,7 +36,7 @@
 #define TAG CLIENT_TAG("sdl")
 
 #include "sdl_monitor.hpp"
-#include "sdl_freerdp.hpp"
+#include "sdl_context.hpp"
 
 typedef struct
 {
@@ -81,7 +81,40 @@ int sdl_list_monitors([[maybe_unused]] SdlContext* sdl)
 	return 0;
 }
 
-static BOOL sdl_apply_max_size(SdlContext* sdl, UINT32* pMaxWidth, UINT32* pMaxHeight)
+[[nodiscard]] static BOOL sdl_apply_mon_max_size(SdlContext* sdl, UINT32* pMaxWidth,
+                                                 UINT32* pMaxHeight)
+{
+	int32_t left = 0;
+	int32_t right = 0;
+	int32_t top = 0;
+	int32_t bottom = 0;
+
+	WINPR_ASSERT(pMaxWidth);
+	WINPR_ASSERT(pMaxHeight);
+
+	auto settings = sdl->context()->settings;
+	WINPR_ASSERT(settings);
+
+	for (size_t x = 0; x < freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount); x++)
+	{
+		auto monitor = static_cast<const rdpMonitor*>(
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x));
+
+		left = std::min(monitor->x, left);
+		top = std::min(monitor->y, top);
+		right = std::max(monitor->x + monitor->width, right);
+		bottom = std::max(monitor->y + monitor->height, bottom);
+	}
+
+	const int32_t w = right - left;
+	const int32_t h = bottom - top;
+
+	*pMaxWidth = WINPR_ASSERTING_INT_CAST(uint32_t, w);
+	*pMaxHeight = WINPR_ASSERTING_INT_CAST(uint32_t, h);
+	return TRUE;
+}
+
+[[nodiscard]] static BOOL sdl_apply_max_size(SdlContext* sdl, UINT32* pMaxWidth, UINT32* pMaxHeight)
 {
 	WINPR_ASSERT(sdl);
 	WINPR_ASSERT(pMaxWidth);
@@ -138,14 +171,15 @@ static BOOL sdl_apply_max_size(SdlContext* sdl, UINT32* pMaxWidth, UINT32* pMaxH
 	return TRUE;
 }
 
-static Uint32 scale(Uint32 val, float scale)
+[[nodiscard]] static Uint32 scale(Uint32 val, float scale)
 {
 	const auto dval = static_cast<float>(val);
 	const auto sval = dval / scale;
 	return static_cast<Uint32>(sval);
 }
 
-static BOOL sdl_apply_monitor_properties(rdpMonitor& monitor, SDL_DisplayID id, bool isPrimary)
+[[nodiscard]] static BOOL sdl_apply_monitor_properties(rdpMonitor& monitor, SDL_DisplayID id,
+                                                       bool isPrimary)
 {
 
 	float dpi = SDL_GetDisplayContentScale(id);
@@ -216,7 +250,7 @@ static BOOL sdl_apply_monitor_properties(rdpMonitor& monitor, SDL_DisplayID id,
 	return TRUE;
 }
 
-static BOOL sdl_apply_display_properties(SdlContext* sdl)
+[[nodiscard]] static BOOL sdl_apply_display_properties(SdlContext* sdl)
 {
 	WINPR_ASSERT(sdl);
 
@@ -253,7 +287,8 @@ static BOOL sdl_apply_display_properties(SdlContext* sdl)
 	                                                     monitors.size());
 }
 
-static BOOL sdl_detect_single_window(SdlContext* sdl, UINT32* pMaxWidth, UINT32* pMaxHeight)
+[[nodiscard]] static BOOL sdl_detect_single_window(SdlContext* sdl, UINT32* pMaxWidth,
+                                                   UINT32* pMaxHeight)
 {
 	WINPR_ASSERT(sdl);
 	WINPR_ASSERT(pMaxWidth);
@@ -285,7 +320,7 @@ static BOOL sdl_detect_single_window(SdlContext* sdl, UINT32* pMaxWidth, UINT32*
 			return FALSE;
 		return sdl_apply_max_size(sdl, pMaxWidth, pMaxHeight);
 	}
-	return TRUE;
+	return sdl_apply_mon_max_size(sdl, pMaxWidth, pMaxHeight);
 }
 
 BOOL sdl_detect_monitors(SdlContext* sdl, UINT32* pMaxWidth, UINT32* pMaxHeight)
