@@ -25,6 +25,6 @@
 
 #include <freerdp/graphics.h>
 
-BOOL sdl_register_pointer(rdpGraphics* graphics);
+[[nodiscard]] BOOL sdl_register_pointer(rdpGraphics* graphics);
 
-BOOL sdl_Pointer_Set_Process(SdlContext* sdl);
+[[nodiscard]] BOOL sdl_Pointer_Set_Process(SdlContext* sdl);
