@@ -716,6 +716,9 @@ static BOOL freerdp_client_rdp_file_set_string(rdpFile* file, const char* name,
 
 	if (targetValue)
 	{
+		if ((uintptr_t)(*targetValue) != UINTPTR_MAX)
+			free(*targetValue);
+
 		*targetValue = _strdup(value);
 		if (!(*targetValue))
 			return FALSE;
@@ -1063,14 +1066,14 @@ BOOL freerdp_client_parse_rdp_file_ex(rdpFile* file, const char* name, rdp_file_
 	return status;
 }
 
-static INLINE void freerdp_client_file_string_reset(char** target)
+static inline void freerdp_client_file_string_reset(char** target)
 {
 	WINPR_ASSERT(target);
 	freerdp_client_file_string_check_free(*target);
 	*target = (void*)~((size_t)NULL);
 }
 
-static INLINE BOOL FILE_POPULATE_STRING(char** _target, const rdpSettings* _settings,
+static inline BOOL FILE_POPULATE_STRING(char** _target, const rdpSettings* _settings,
                                         FreeRDP_Settings_Keys_String _option)
 {
 	WINPR_ASSERT(_target);
@@ -1957,7 +1960,8 @@ BOOL freerdp_client_populate_settings_from_rdp_file_unchecked(const rdpFile* fil
 
 	if (~file->EnableRdsAadAuth)
 	{
-		if (!freerdp_settings_set_bool(settings, FreeRDP_AadSecurity, file->EnableRdsAadAuth != 0))
+		const BOOL val = file->EnableRdsAadAuth != 0;
+		if (!freerdp_settings_set_bool(settings, FreeRDP_AadSecurity, val))
 			return FALSE;
 	}
 
