@@ -300,19 +300,26 @@ static inline RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
 	tile->height = 64;
 	tile->stride = 4 * tile->width;
 
-	size_t dataLen = 1ull * tile->stride * tile->height;
-	tile->data = (BYTE*)winpr_aligned_malloc(dataLen, 16);
-	if (!tile->data)
-		goto fail;
-	memset(tile->data, 0xFF, dataLen);
+	{
+		const size_t dataLen = 1ull * tile->stride * tile->height;
+		tile->data = (BYTE*)winpr_aligned_malloc(dataLen, 16);
+		if (!tile->data)
+			goto fail;
+		memset(tile->data, 0xFF, dataLen);
+	}
+
+	{
+		const size_t signLen = (8192ULL + 32ULL) * 3ULL;
+		tile->sign = (BYTE*)winpr_aligned_malloc(signLen, 16);
+	}
 
-	size_t signLen = (8192ULL + 32ULL) * 3ULL;
-	tile->sign = (BYTE*)winpr_aligned_malloc(signLen, 16);
 	if (!tile->sign)
 		goto fail;
 
-	size_t currentLen = (8192ULL + 32ULL) * 3ULL;
-	tile->current = (BYTE*)winpr_aligned_malloc(currentLen, 16);
+	{
+		const size_t currentLen = (8192ULL + 32ULL) * 3ULL;
+		tile->current = (BYTE*)winpr_aligned_malloc(currentLen, 16);
+	}
 	if (!tile->current)
 		goto fail;
 
@@ -965,9 +972,11 @@ progressive_decompress_tile_first(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressiv
 	if (rc < 0)
 		goto fail;
 
-	const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
-	rc = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride, progressive->format,
-	                                    &roi_64x64);
+	{
+		const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
+		rc = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
+		                                    progressive->format, &roi_64x64);
+	}
 fail:
 	BufferPool_Return(progressive->bufferPool, pBuffer);
 	return rc;
@@ -1090,13 +1099,25 @@ progressive_rfx_upgrade_state_finish(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRI
 	return 1;
 }
 
+static inline int16_t rawShift(wBitStream* raw, UINT32 numBits)
+{
+	WINPR_ASSERT(raw);
+	WINPR_ASSERT(numBits > 0);
+
+	raw->mask = ((1 << numBits) - 1);
+	BitStream_Shift(raw, numBits);
+	const unsigned input = ((raw->accumulator >> (32 - numBits)) & raw->mask);
+	int16_t val = (int16_t)input;
+	return val;
+}
+
 static inline int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
                                                 INT16* WINPR_RESTRICT buffer,
                                                 INT16* WINPR_RESTRICT sign, UINT32 length,
                                                 UINT32 shift, WINPR_ATTR_UNUSED UINT32 bitPos,
                                                 UINT32 numBits)
 {
-	if (!numBits)
+	if (numBits < 1)
 		return 1;
 
 	wBitStream* raw = state->raw;
@@ -1106,9 +1127,7 @@ static inline int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* W
 	{
 		for (UINT32 index = 0; index < length; index++)
 		{
-			raw->mask = ((1 << numBits) - 1);
-			input = (INT16)((raw->accumulator >> (32 - numBits)) & raw->mask);
-			BitStream_Shift(raw, numBits);
+			input = rawShift(raw, numBits);
 
 			const int32_t shifted = input << shift;
 			const int32_t val = buffer[index] + shifted;
@@ -1124,26 +1143,25 @@ static inline int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* W
 		if (sign[index] > 0)
 		{
 			/* sign > 0, read from raw */
-			raw->mask = ((1 << numBits) - 1);
-			input = (INT16)((raw->accumulator >> (32 - numBits)) & raw->mask);
-			BitStream_Shift(raw, numBits);
+			input = rawShift(raw, numBits);
 		}
 		else if (sign[index] < 0)
 		{
 			/* sign < 0, read from raw */
-			raw->mask = ((1 << numBits) - 1);
-			input = (INT16)((raw->accumulator >> (32 - numBits)) & raw->mask);
-			BitStream_Shift(raw, numBits);
-			input *= -1;
+			input = rawShift(raw, numBits);
 		}
 		else
 		{
 			/* sign == 0, read from srl */
 			input = progressive_rfx_srl_read(state, numBits);
 			sign[index] = WINPR_ASSERTING_INT_CAST(int16_t, input);
+			if (sign[index] < 0)
+				input *= -1;
 		}
 
-		const int32_t val = input << shift;
+		int32_t val = input << shift;
+		if (sign[index] < 0)
+			val *= -1;
 		const int32_t ival = buffer[index] + val;
 		buffer[index] = WINPR_ASSERTING_INT_CAST(INT16, ival);
 	}
@@ -1417,9 +1435,11 @@ progressive_decompress_tile_upgrade(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progress
 	if (status < 0)
 		goto fail;
 
-	const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
-	status = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
-	                                        progressive->format, &roi_64x64);
+	{
+		const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
+		status = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
+		                                        progressive->format, &roi_64x64);
+	}
 fail:
 	BufferPool_Return(progressive->bufferPool, pBuffer);
 	return status;
@@ -2366,14 +2386,16 @@ INT32 progressive_decompress(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 			goto fail;
 	}
 
-	const size_t end = Stream_GetPosition(s);
-	if ((end - start) != SrcSize)
 	{
-		WLog_Print(progressive->log, WLOG_ERROR,
-		           "total block len %" PRIuz " does not match read data %" PRIu32, end - start,
-		           SrcSize);
-		rc = -1041;
-		goto fail;
+		const size_t end = Stream_GetPosition(s);
+		if ((end - start) != SrcSize)
+		{
+			WLog_Print(progressive->log, WLOG_ERROR,
+			           "total block len %" PRIuz " does not match read data %" PRIu32, end - start,
+			           SrcSize);
+			rc = -1041;
+			goto fail;
+		}
 	}
 
 	if (!update_tiles(progressive, surface, pDstData, DstFormat, nDstStep, nXDst, nYDst, region,
@@ -2518,9 +2540,11 @@ int progressive_compress(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 	if (!rc)
 		goto fail;
 
-	const size_t pos = Stream_GetPosition(s);
-	WINPR_ASSERT(pos <= UINT32_MAX);
-	*pDstSize = (UINT32)pos;
+	{
+		const size_t pos = Stream_GetPosition(s);
+		WINPR_ASSERT(pos <= UINT32_MAX);
+		*pDstSize = (UINT32)pos;
+	}
 	*ppDstData = Stream_Buffer(s);
 	res = 1;
 fail:
