@@ -710,7 +710,9 @@ static BOOL autodetect_recv_netchar_sync(rdpAutoDetect* autodetect, RDP_TRANSPOR
 static BOOL autodetect_recv_netchar_request(rdpAutoDetect* autodetect, RDP_TRANSPORT_TYPE transport,
                                             wStream* s, const AUTODETECT_REQ_PDU* autodetectReqPdu)
 {
-	rdpNetworkCharacteristicsResult result = { 0 };
+	rdpNetworkCharacteristicsResult result = {
+		.type = RDP_NETCHAR_RESERVED, .baseRTT = 0, .averageRTT = 0, .bandwidth = 0
+	};
 	BOOL success = TRUE;
 
 	WINPR_ASSERT(autodetect);
