@@ -37,8 +37,8 @@
 
 #define TAG FREERDP_TAG("core.info")
 
-#define logonInfoV2Size (2 + 4 + 4 + 4 + 4)
-#define logonInfoV2ReservedSize 558
+#define logonInfoV2Size (2u + 4u + 4u + 4u + 4u)
+#define logonInfoV2ReservedSize 558u
 #define logonInfoV2TotalSize (logonInfoV2Size + logonInfoV2ReservedSize)
 
 static const char* INFO_TYPE_LOGON_STRINGS[4] = { "Logon Info V1", "Logon Info V2",
@@ -436,7 +436,7 @@ static BOOL rdp_read_extended_info_packet(rdpRdp* rdp, wStream* s)
 		{
 			WLog_WARN(TAG,
 			          "[MS-RDPBCGR] 2.2.1.11.1.1.1 Extended Info Packet "
-			          "(TS_EXTENDED_INFO_PACKET)::dynamicDaylightTimeDisabled value %" PRIu16
+			          "(TS_EXTENDED_INFO_PACKET)::dynamicDaylightTimeDisabled value %d"
 			          " not allowed in [0,1]",
 			          settings->DynamicDaylightTimeDisabled);
 			return FALSE;
@@ -1067,7 +1067,7 @@ static BOOL rdp_info_read_string(const char* what, wStream* s, size_t size, size
 
 	if (((size % sizeof(WCHAR)) != 0) || (size > max))
 	{
-		WLog_ERR(TAG, "protocol error: invalid %s value: %" PRIu32 "", what, size);
+		WLog_ERR(TAG, "protocol error: invalid %s value: %" PRIuz "", what, size);
 		return FALSE;
 	}
 
@@ -1148,7 +1148,7 @@ static BOOL rdp_recv_logon_info_v2(rdpRdp* rdp, wStream* s, logon_info* info)
 	Stream_Read_UINT16(s, Version); /* Version (2 bytes) */
 	if (Version != SAVE_SESSION_PDU_VERSION_ONE)
 	{
-		WLog_WARN(TAG, "LogonInfoV2::Version expected %" PRIu16 " bytes, got %" PRIu16,
+		WLog_WARN(TAG, "LogonInfoV2::Version expected %d bytes, got %" PRIu16,
 		          SAVE_SESSION_PDU_VERSION_ONE, Version);
 		return FALSE;
 	}
@@ -1209,7 +1209,7 @@ static BOOL rdp_recv_logon_info_v2(rdpRdp* rdp, wStream* s, logon_info* info)
 			}
 			if (warn)
 			{
-				WLog_WARN(TAG, "unexpected padding of %" PRIuz " bytes, data not '\0'", rem);
+				WLog_WARN(TAG, "unexpected padding of %" PRIuz " bytes, data not '\\0'", rem);
 				winpr_HexDump(TAG, WLOG_TRACE, str, rem);
 			}
 
@@ -1447,7 +1447,7 @@ static BOOL rdp_write_logon_info_v1(wStream* s, logon_info* info)
 	return TRUE;
 }
 
-static BOOL rdp_write_logon_info_v2(wStream* s, logon_info* info)
+static BOOL rdp_write_logon_info_v2(wStream* s, const logon_info* info)
 {
 	size_t domainLen = 0;
 	size_t usernameLen = 0;
@@ -1462,11 +1462,14 @@ static BOOL rdp_write_logon_info_v2(wStream* s, logon_info* info)
 	 */
 	Stream_Write_UINT32(s, logonInfoV2Size);
 	Stream_Write_UINT32(s, info->sessionId);
-	domainLen = strnlen(info->domain, 256); /* lmcons.h UNLEN */
+	if (info->domain)
+		domainLen = strnlen(info->domain, 256); /* lmcons.h UNLEN */
 	if (domainLen >= UINT32_MAX / sizeof(WCHAR))
 		return FALSE;
 	Stream_Write_UINT32(s, (UINT32)(domainLen + 1) * sizeof(WCHAR));
-	usernameLen = strnlen(info->username, 256); /* lmcons.h UNLEN */
+
+	if (info->username)
+		usernameLen = strnlen(info->username, 256); /* lmcons.h UNLEN */
 	if (usernameLen >= UINT32_MAX / sizeof(WCHAR))
 		return FALSE;
 	Stream_Write_UINT32(s, (UINT32)(usernameLen + 1) * sizeof(WCHAR));
@@ -1534,10 +1537,11 @@ static BOOL rdp_write_logon_info_ex(wStream* s, logon_info_ex* info)
 BOOL rdp_send_save_session_info(rdpContext* context, UINT32 type, void* data)
 {
 	UINT16 sec_flags = 0;
-	wStream* s = NULL;
 	BOOL status = 0;
+
+	WINPR_ASSERT(context);
 	rdpRdp* rdp = context->rdp;
-	s = rdp_data_pdu_init(rdp, &sec_flags);
+	wStream* s = rdp_data_pdu_init(rdp, &sec_flags);
 
 	if (!s)
 		return FALSE;
