@@ -24,9 +24,9 @@
 #include <openssl/bio.h>
 
 BOOL proxy_prepare(rdpSettings* settings, const char** lpPeerHostname, UINT16* lpPeerPort,
-					const char **lpProxyUsername, const char **lpProxyPassword);
-BOOL proxy_parse_uri(rdpSettings* settings, const char* uri);
-BOOL proxy_connect(rdpSettings* settings, BIO* bio, const char *proxyUsername, const char *proxyPassword,
-		const char* hostname, UINT16 port);
+                   const char** lpProxyUsername, const char** lpProxyPassword);
+
+BOOL proxy_connect(rdpContext* context, BIO* bio, const char* proxyUsername,
+                   const char* proxyPassword, const char* hostname, UINT16 port);
 
 #endif /* FREERDP_LIB_CORE_HTTP_PROXY_H */
