@@ -26,14 +26,17 @@
 #include <unistd.h>
 #include <assert.h>
 #include <sys/mman.h>
+#include <errno.h>
 
 #include "uwac-priv.h"
 #include "uwac-utils.h"
 #include "uwac-os.h"
 
+#include <uwac/config.h>
 
-#define UWAC_INITIAL_BUFFERS 3
+#include <winpr/cast.h>
 
+#define UWAC_INITIAL_BUFFERS 3ull
 
 static int bppFromShmFormat(enum wl_shm_format format)
 {
@@ -46,31 +49,30 @@ static int bppFromShmFormat(enum wl_shm_format format)
 	}
 }
 
-
 static void buffer_release(void* data, struct wl_buffer* buffer)
 {
-	UwacBuffer* uwacBuffer = (UwacBuffer*)data;
+	UwacBufferReleaseData* releaseData = data;
+	UwacBuffer* uwacBuffer = &releaseData->window->buffers[releaseData->bufferIdx];
 	uwacBuffer->used = false;
 }
 
-static const struct wl_buffer_listener buffer_listener =
-{
-	buffer_release
-};
+static const struct wl_buffer_listener buffer_listener = { buffer_release };
 
-void UwacWindowDestroyBuffers(UwacWindow* w)
+static void UwacWindowDestroyBuffers(UwacWindow* w)
 {
-	int i;
-
-	for (i = 0; i < w->nbuffers; i++)
+	for (size_t i = 0; i < w->nbuffers; i++)
 	{
 		UwacBuffer* buffer = &w->buffers[i];
-#ifdef HAVE_PIXMAN_REGION
+#ifdef UWAC_HAVE_PIXMAN_REGION
 		pixman_region32_fini(&buffer->damage);
 #else
 		region16_uninit(&buffer->damage);
 #endif
+		UwacBufferReleaseData* releaseData =
+		    (UwacBufferReleaseData*)wl_buffer_get_user_data(buffer->wayland_buffer);
 		wl_buffer_destroy(buffer->wayland_buffer);
+		free(releaseData);
+		munmap(buffer->data, buffer->size);
 	}
 
 	w->nbuffers = 0;
@@ -78,20 +80,22 @@ void UwacWindowDestroyBuffers(UwacWindow* w)
 	w->buffers = NULL;
 }
 
+static int UwacWindowShmAllocBuffers(UwacWindow* w, uint64_t nbuffers, uint64_t allocSize,
+                                     uint32_t width, uint32_t height, enum wl_shm_format format);
 
-int UwacWindowShmAllocBuffers(UwacWindow* w, int nbuffers, int allocSize, uint32_t width,
-                              uint32_t height, enum wl_shm_format format);
-
-static void xdg_handle_configure(void *data,
-                                 struct xdg_toplevel *xdg_toplevel,
-                                 int32_t width,
-                                 int32_t height,
-                                 struct wl_array *states)
+static void xdg_handle_toplevel_configure(void* data, struct xdg_toplevel* xdg_toplevel,
+                                          int32_t width, int32_t height, struct wl_array* states)
 {
 	UwacWindow* window = (UwacWindow*)data;
-	UwacConfigureEvent* event;
-	int ret, surfaceState;
-	enum xdg_toplevel_state* state;
+	int scale = window->display->actual_scale;
+	int32_t actual_width = width;
+	int32_t actual_height = height;
+	width *= scale;
+	height *= scale;
+	UwacConfigureEvent* event = NULL;
+	int ret = 0;
+	int surfaceState = 0;
+	enum xdg_toplevel_state* state = NULL;
 	surfaceState = 0;
 	wl_array_for_each(state, states)
 	{
@@ -130,7 +134,7 @@ static void xdg_handle_configure(void *data,
 	event->window = window;
 	event->states = surfaceState;
 
-	if (width && height)
+	if ((width > 0 && height > 0) && (width != window->width || height != window->height))
 	{
 		event->width = width;
 		event->height = height;
@@ -138,17 +142,29 @@ static void xdg_handle_configure(void *data,
 		window->width = width;
 		window->stride = width * bppFromShmFormat(window->format);
 		window->height = height;
-		ret = UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, window->stride * height,
-		                                width, height, window->format);
+		ret =
+		    UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, 1ull * window->stride * height,
+		                              width, height, window->format);
 
 		if (ret != UWAC_SUCCESS)
 		{
-			assert(uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
-			window->drawingBuffer = window->pendingBuffer = NULL;
+			assert(
+			    uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
+			window->drawingBufferIdx = window->pendingBufferIdx = -1;
 			return;
 		}
 
-		window->drawingBuffer = window->pendingBuffer = &window->buffers[0];
+		window->drawingBufferIdx = 0;
+		if (window->pendingBufferIdx != -1)
+			window->pendingBufferIdx = window->drawingBufferIdx;
+
+		if (window->viewport)
+		{
+			wp_viewport_set_source(window->viewport, wl_fixed_from_int(0), wl_fixed_from_int(0),
+			                       wl_fixed_from_int(actual_width),
+			                       wl_fixed_from_int(actual_height));
+			wp_viewport_set_destination(window->viewport, actual_width, actual_height);
+		}
 	}
 	else
 	{
@@ -157,10 +173,9 @@ static void xdg_handle_configure(void *data,
 	}
 }
 
-static void xdg_handle_close(void *data,
-                             struct xdg_toplevel *xdg_toplevel)
+static void xdg_handle_toplevel_close(void* data, struct xdg_toplevel* xdg_toplevel)
 {
-	UwacCloseEvent* event;
+	UwacCloseEvent* event = NULL;
 	UwacWindow* window = (UwacWindow*)data;
 	event = (UwacCloseEvent*)UwacDisplayNewEvent(window->display, UWAC_EVENT_CLOSE);
 
@@ -174,19 +189,29 @@ static void xdg_handle_close(void *data,
 	event->window = window;
 }
 
-static const struct xdg_toplevel_listener xdg_toplevel_listener =
-{
-	xdg_handle_configure,
-	xdg_handle_close,
+static const struct xdg_toplevel_listener xdg_toplevel_listener = {
+	xdg_handle_toplevel_configure,
+	xdg_handle_toplevel_close,
 };
+
+static void xdg_handle_surface_configure(void* data, struct xdg_surface* xdg_surface,
+                                         uint32_t serial)
+{
+	xdg_surface_ack_configure(xdg_surface, serial);
+}
+
+static const struct xdg_surface_listener xdg_surface_listener = {
+	.configure = xdg_handle_surface_configure,
+};
+
 #if BUILD_IVI
 
-static void ivi_handle_configure(void* data, struct ivi_surface* surface,
-                                 int32_t width, int32_t height)
+static void ivi_handle_configure(void* data, struct ivi_surface* surface, int32_t width,
+                                 int32_t height)
 {
 	UwacWindow* window = (UwacWindow*)data;
-	UwacConfigureEvent* event;
-	int ret;
+	UwacConfigureEvent* event = NULL;
+	int ret = 0;
 	event = (UwacConfigureEvent*)UwacDisplayNewEvent(window->display, UWAC_EVENT_CONFIGURE);
 
 	if (!event)
@@ -207,17 +232,21 @@ static void ivi_handle_configure(void* data, struct ivi_surface* surface,
 		window->width = width;
 		window->stride = width * bppFromShmFormat(window->format);
 		window->height = height;
-		ret = UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, window->stride * height,
-		                                width, height, window->format);
+		ret =
+		    UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, 1ull * window->stride * height,
+		                              width, height, window->format);
 
 		if (ret != UWAC_SUCCESS)
 		{
-			assert(uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
-			window->drawingBuffer = window->pendingBuffer = NULL;
+			assert(
+			    uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
+			window->drawingBufferIdx = window->pendingBufferIdx = -1;
 			return;
 		}
 
-		window->drawingBuffer = window->pendingBuffer = &window->buffers[0];
+		window->drawingBufferIdx = 0;
+		if (window->pendingBufferIdx != -1)
+			window->pendingBufferIdx = window->drawingBufferIdx;
 	}
 	else
 	{
@@ -226,23 +255,22 @@ static void ivi_handle_configure(void* data, struct ivi_surface* surface,
 	}
 }
 
-static const struct ivi_surface_listener ivi_surface_listener =
-{
+static const struct ivi_surface_listener ivi_surface_listener = {
 	ivi_handle_configure,
 };
 #endif
 
-void shell_ping(void* data, struct wl_shell_surface* surface, uint32_t serial)
+static void shell_ping(void* data, struct wl_shell_surface* surface, uint32_t serial)
 {
 	wl_shell_surface_pong(surface, serial);
 }
 
-void shell_configure(void* data, struct wl_shell_surface* surface, uint32_t edges,
-                     int32_t width, int32_t height)
+static void shell_configure(void* data, struct wl_shell_surface* surface, uint32_t edges,
+                            int32_t width, int32_t height)
 {
 	UwacWindow* window = (UwacWindow*)data;
-	UwacConfigureEvent* event;
-	int ret;
+	UwacConfigureEvent* event = NULL;
+	int ret = 0;
 	event = (UwacConfigureEvent*)UwacDisplayNewEvent(window->display, UWAC_EVENT_CONFIGURE);
 
 	if (!event)
@@ -263,17 +291,21 @@ void shell_configure(void* data, struct wl_shell_surface* surface, uint32_t edge
 		window->width = width;
 		window->stride = width * bppFromShmFormat(window->format);
 		window->height = height;
-		ret = UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, window->stride * height,
-		                                width, height, window->format);
+		ret =
+		    UwacWindowShmAllocBuffers(window, UWAC_INITIAL_BUFFERS, 1ull * window->stride * height,
+		                              width, height, window->format);
 
 		if (ret != UWAC_SUCCESS)
 		{
-			assert(uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
-			window->drawingBuffer = window->pendingBuffer = NULL;
+			assert(
+			    uwacErrorHandler(window->display, ret, "failed to reallocate a wayland buffers\n"));
+			window->drawingBufferIdx = window->pendingBufferIdx = -1;
 			return;
 		}
 
-		window->drawingBuffer = window->pendingBuffer = &window->buffers[0];
+		window->drawingBufferIdx = 0;
+		if (window->pendingBufferIdx != -1)
+			window->pendingBufferIdx = window->drawingBufferIdx;
 	}
 	else
 	{
@@ -282,43 +314,56 @@ void shell_configure(void* data, struct wl_shell_surface* surface, uint32_t edge
 	}
 }
 
-
-void shell_popup_done(void* data, struct wl_shell_surface* surface)
+static void shell_popup_done(void* data, struct wl_shell_surface* surface)
 {
 }
 
+static const struct wl_shell_surface_listener shell_listener = { shell_ping, shell_configure,
+	                                                             shell_popup_done };
 
-static const struct wl_shell_surface_listener shell_listener =
-{
-	shell_ping,
-	shell_configure,
-	shell_popup_done
-};
-
-
-int UwacWindowShmAllocBuffers(UwacWindow* w, int nbuffers, int allocSize, uint32_t width,
+int UwacWindowShmAllocBuffers(UwacWindow* w, uint64_t nbuffers, uint64_t allocSize, uint32_t width,
                               uint32_t height, enum wl_shm_format format)
 {
 	int ret = UWAC_SUCCESS;
-	UwacBuffer* newBuffers;
-	int i, fd;
-	void* data;
-	struct wl_shm_pool* pool;
-	newBuffers = realloc(w->buffers, (w->nbuffers + nbuffers) * sizeof(UwacBuffer));
+	int fd = 0;
+	void* data = NULL;
+	struct wl_shm_pool* pool = NULL;
+
+	if ((width > INT32_MAX) || (height > INT32_MAX))
+		return UWAC_ERROR_NOMEMORY;
+
+	const int64_t pagesize = sysconf(_SC_PAGESIZE);
+	if (pagesize <= 0)
+		return UWAC_ERROR_NOMEMORY;
+
+	/* round up to a multiple of PAGESIZE to page align data for each buffer */
+	const uint64_t test = (1ull * allocSize + (size_t)pagesize - 1ull) & ~((size_t)pagesize - 1);
+	if (test > INT64_MAX)
+		return UWAC_ERROR_NOMEMORY;
+
+	allocSize = test;
+
+	UwacBuffer* newBuffers =
+	    xrealloc(w->buffers, (0ull + w->nbuffers + nbuffers) * sizeof(UwacBuffer));
 
 	if (!newBuffers)
 		return UWAC_ERROR_NOMEMORY;
 
 	w->buffers = newBuffers;
 	memset(w->buffers + w->nbuffers, 0, sizeof(UwacBuffer) * nbuffers);
-	fd = uwac_create_anonymous_file(allocSize * nbuffers);
+
+	const size_t allocbuffersize = 1ull * allocSize * nbuffers;
+	if (allocbuffersize > INT32_MAX)
+		return UWAC_ERROR_NOMEMORY;
+
+	fd = uwac_create_anonymous_file(WINPR_ASSERTING_INT_CAST(off_t, allocbuffersize));
 
 	if (fd < 0)
 	{
 		return UWAC_ERROR_INTERNAL;
 	}
 
-	data = mmap(NULL, allocSize * nbuffers, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+	data = mmap(NULL, allocbuffersize, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 
 	if (data == MAP_FAILED)
 	{
@@ -326,27 +371,38 @@ int UwacWindowShmAllocBuffers(UwacWindow* w, int nbuffers, int allocSize, uint32
 		goto error_mmap;
 	}
 
-	pool = wl_shm_create_pool(w->display->shm, fd, allocSize * nbuffers);
+	pool = wl_shm_create_pool(w->display->shm, fd, (int32_t)allocbuffersize);
 
 	if (!pool)
 	{
-		munmap(data, allocSize * nbuffers);
+		munmap(data, allocbuffersize);
 		ret = UWAC_ERROR_NOMEMORY;
 		goto error_mmap;
 	}
 
-	for (i = 0; i < nbuffers; i++)
+	for (uint64_t i = 0; i < nbuffers; i++)
 	{
-		UwacBuffer* buffer = &w->buffers[w->nbuffers + i];
-#ifdef HAVE_PIXMAN_REGION
+		const size_t idx = (size_t)i;
+		const size_t bufferIdx = w->nbuffers + idx;
+		UwacBuffer* buffer = &w->buffers[bufferIdx];
+
+#ifdef UWAC_HAVE_PIXMAN_REGION
 		pixman_region32_init(&buffer->damage);
 #else
 		region16_init(&buffer->damage);
 #endif
-		buffer->data = data + (allocSize * i);
-		buffer->wayland_buffer = wl_shm_pool_create_buffer(pool, allocSize * i, width, height, w->stride,
-		                         format);
-		wl_buffer_add_listener(buffer->wayland_buffer, &buffer_listener, buffer);
+		const size_t offset = allocSize * idx;
+		if (offset > INT32_MAX)
+			goto error_mmap;
+
+		buffer->data = &((char*)data)[allocSize * idx];
+		buffer->size = allocSize;
+		buffer->wayland_buffer = wl_shm_pool_create_buffer(pool, (int32_t)offset, (int32_t)width,
+		                                                   (int32_t)height, w->stride, format);
+		UwacBufferReleaseData* listener_data = xmalloc(sizeof(UwacBufferReleaseData));
+		listener_data->window = w;
+		listener_data->bufferIdx = bufferIdx;
+		wl_buffer_add_listener(buffer->wayland_buffer, &buffer_listener, listener_data);
 	}
 
 	wl_shm_pool_destroy(pool);
@@ -356,20 +412,27 @@ error_mmap:
 	return ret;
 }
 
-UwacBuffer* UwacWindowFindFreeBuffer(UwacWindow* w)
+static UwacBuffer* UwacWindowFindFreeBuffer(UwacWindow* w, ssize_t* index)
 {
-	int i, ret;
+	int ret = 0;
 
-	for (i = 0; i < w->nbuffers; i++)
+	if (index)
+		*index = -1;
+
+	size_t i = 0;
+	for (; i < w->nbuffers; i++)
 	{
 		if (!w->buffers[i].used)
 		{
 			w->buffers[i].used = true;
+			if (index)
+				*index = WINPR_ASSERTING_INT_CAST(ssize_t, i);
 			return &w->buffers[i];
 		}
 	}
 
-	ret = UwacWindowShmAllocBuffers(w, 2, w->stride * w->height, w->width, w->height, w->format);
+	ret = UwacWindowShmAllocBuffers(w, 2, 1ull * w->stride * w->height, w->width, w->height,
+	                                w->format);
 
 	if (ret != UWAC_SUCCESS)
 	{
@@ -378,30 +441,41 @@ UwacBuffer* UwacWindowFindFreeBuffer(UwacWindow* w)
 	}
 
 	w->buffers[i].used = true;
+	if (index)
+		*index = WINPR_ASSERTING_INT_CAST(ssize_t, i);
 	return &w->buffers[i];
 }
 
-static UwacReturnCode UwacWindowSetDecorations(UwacWindow *w)
+static UwacReturnCode UwacWindowSetDecorations(UwacWindow* w)
 {
 	if (!w || !w->display)
 		return UWAC_ERROR_INTERNAL;
 
-	if (w->display->deco_manager) {
-		w->deco = zxdg_decoration_manager_v1_get_toplevel_decoration(
-		            w->display->deco_manager, w->xdg_toplevel);
-		if (!w->deco) {
-			uwacErrorHandler(w->display, UWAC_NOT_FOUND, "Current window manager does not allow decorating with SSD");
+	if (w->display->deco_manager)
+	{
+		w->deco = zxdg_decoration_manager_v1_get_toplevel_decoration(w->display->deco_manager,
+		                                                             w->xdg_toplevel);
+		if (!w->deco)
+		{
+			uwacErrorHandler(w->display, UWAC_NOT_FOUND,
+			                 "Current window manager does not allow decorating with SSD");
 		}
 		else
-			zxdg_toplevel_decoration_v1_set_mode(w->deco, ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
+			zxdg_toplevel_decoration_v1_set_mode(w->deco,
+			                                     ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
 	}
-	else if (w->display->kde_deco_manager) {
-		w->kde_deco = org_kde_kwin_server_decoration_manager_create(w->display->kde_deco_manager, w->surface);
-		if (!w->kde_deco) {
-			uwacErrorHandler(w->display, UWAC_NOT_FOUND, "Current window manager does not allow decorating with SSD");
+	else if (w->display->kde_deco_manager)
+	{
+		w->kde_deco =
+		    org_kde_kwin_server_decoration_manager_create(w->display->kde_deco_manager, w->surface);
+		if (!w->kde_deco)
+		{
+			uwacErrorHandler(w->display, UWAC_NOT_FOUND,
+			                 "Current window manager does not allow decorating with SSD");
 		}
 		else
-			org_kde_kwin_server_decoration_request_mode(w->kde_deco, ORG_KDE_KWIN_SERVER_DECORATION_MODE_SERVER);
+			org_kde_kwin_server_decoration_request_mode(w->kde_deco,
+			                                            ORG_KDE_KWIN_SERVER_DECORATION_MODE_SERVER);
 	}
 	return UWAC_SUCCESS;
 }
@@ -409,15 +483,15 @@ static UwacReturnCode UwacWindowSetDecorations(UwacWindow *w)
 UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t height,
                                 enum wl_shm_format format)
 {
-	UwacWindow* w;
-	int allocSize, ret;
+	UwacWindow* w = NULL;
+	int ret = 0;
 
 	if (!display)
 	{
 		return NULL;
 	}
 
-	w = zalloc(sizeof(*w));
+	w = xzalloc(sizeof(*w));
 
 	if (!w)
 	{
@@ -427,10 +501,10 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 
 	w->display = display;
 	w->format = format;
-	w->width = width;
-	w->height = height;
-	w->stride = width * bppFromShmFormat(format);
-	allocSize = w->stride * height;
+	w->width = WINPR_ASSERTING_INT_CAST(int32_t, width);
+	w->height = WINPR_ASSERTING_INT_CAST(int32_t, height);
+	w->stride = WINPR_ASSERTING_INT_CAST(int32_t, width* bppFromShmFormat(format));
+	const size_t allocSize = 1ULL * w->stride * height;
 	ret = UwacWindowShmAllocBuffers(w, UWAC_INITIAL_BUFFERS, allocSize, width, height, format);
 
 	if (ret != UWAC_SUCCESS)
@@ -440,7 +514,8 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 	}
 
 	w->buffers[0].used = true;
-	w->drawingBuffer = &w->buffers[0];
+	w->drawingBufferIdx = 0;
+	w->pendingBufferIdx = -1;
 	w->surface = wl_compositor_create_surface(display->compositor);
 
 	if (!w->surface)
@@ -451,7 +526,41 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 
 	wl_surface_set_user_data(w->surface, w);
 
-	if (display->xdg_base)
+#if BUILD_IVI
+	uint32_t ivi_surface_id = 1;
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
+	char* env = getenv("IVI_SURFACE_ID");
+	if (env)
+	{
+		unsigned long val = 0;
+		char* endp = NULL;
+
+		errno = 0;
+		val = strtoul(env, &endp, 10);
+
+		if (!errno && val != 0 && val != UINT32_MAX)
+			ivi_surface_id = val;
+	}
+
+	if (display->ivi_application)
+	{
+		w->ivi_surface =
+		    ivi_application_surface_create(display->ivi_application, ivi_surface_id, w->surface);
+		assert(w->ivi_surface);
+		ivi_surface_add_listener(w->ivi_surface, &ivi_surface_listener, w);
+	}
+	else
+#endif
+#if BUILD_FULLSCREEN_SHELL
+	    if (display->fullscreen_shell)
+	{
+		zwp_fullscreen_shell_v1_present_surface(display->fullscreen_shell, w->surface,
+		                                        ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_CENTER,
+		                                        NULL);
+	}
+	else
+#endif
+	    if (display->xdg_base)
 	{
 		w->xdg_surface = xdg_wm_base_get_xdg_surface(display->xdg_base, w->surface);
 
@@ -461,6 +570,8 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 			goto out_error_shell;
 		}
 
+		xdg_surface_add_listener(w->xdg_surface, &xdg_surface_listener, w);
+
 		w->xdg_toplevel = xdg_surface_get_toplevel(w->xdg_surface);
 		if (!w->xdg_toplevel)
 		{
@@ -470,22 +581,9 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 
 		assert(w->xdg_surface);
 		xdg_toplevel_add_listener(w->xdg_toplevel, &xdg_toplevel_listener, w);
+		wl_surface_commit(w->surface);
+		wl_display_roundtrip(w->display->display);
 	}
-#if BUILD_IVI
-	else if (display->ivi_application)
-	{
-		w->ivi_surface = ivi_application_surface_create(display->ivi_application, 1, w->surface);
-		assert(w->ivi_surface);
-		ivi_surface_add_listener(w->ivi_surface, &ivi_surface_listener, w);
-	}
-#endif
-#if BUILD_FULLSCREEN_SHELL
-	else if (display->fullscreen_shell)
-	{
-		zwp_fullscreen_shell_v1_present_surface(display->fullscreen_shell, w->surface,
-		                                     ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_CENTER, NULL);
-	}
-#endif
 	else
 	{
 		w->shell_surface = wl_shell_get_shell_surface(display->shell, w->surface);
@@ -494,6 +592,13 @@ UwacWindow* UwacCreateWindowShm(UwacDisplay* display, uint32_t width, uint32_t h
 		wl_shell_surface_set_toplevel(w->shell_surface);
 	}
 
+	if (display->viewporter)
+	{
+		w->viewport = wp_viewporter_get_viewport(display->viewporter, w->surface);
+		if (display->actual_scale != 1)
+			wl_surface_set_buffer_scale(w->surface, display->actual_scale);
+	}
+
 	wl_list_insert(display->windows.prev, &w->link);
 	display->last_error = UWAC_SUCCESS;
 	UwacWindowSetDecorations(w);
@@ -507,10 +612,9 @@ out_error_free:
 	return NULL;
 }
 
-
 UwacReturnCode UwacDestroyWindow(UwacWindow** pwindow)
 {
-	UwacWindow* w;
+	UwacWindow* w = NULL;
 	assert(pwindow);
 	w = *pwindow;
 	UwacWindowDestroyBuffers(w);
@@ -537,6 +641,9 @@ UwacReturnCode UwacDestroyWindow(UwacWindow** pwindow)
 	if (w->input_region)
 		wl_region_destroy(w->input_region);
 
+	if (w->viewport)
+		wp_viewport_destroy(w->viewport);
+
 	wl_surface_destroy(w->surface);
 	wl_list_remove(&w->link);
 	free(w);
@@ -544,9 +651,8 @@ UwacReturnCode UwacDestroyWindow(UwacWindow** pwindow)
 	return UWAC_SUCCESS;
 }
 
-
 UwacReturnCode UwacWindowSetOpaqueRegion(UwacWindow* window, uint32_t x, uint32_t y, uint32_t width,
-        uint32_t height)
+                                         uint32_t height)
 {
 	assert(window);
 
@@ -558,7 +664,9 @@ UwacReturnCode UwacWindowSetOpaqueRegion(UwacWindow* window, uint32_t x, uint32_
 	if (!window->opaque_region)
 		return UWAC_ERROR_NOMEMORY;
 
-	wl_region_add(window->opaque_region, x, y, width, height);
+	wl_region_add(window->opaque_region, WINPR_ASSERTING_INT_CAST(int32_t, x),
+	              WINPR_ASSERTING_INT_CAST(int32_t, y), WINPR_ASSERTING_INT_CAST(int32_t, width),
+	              WINPR_ASSERTING_INT_CAST(int32_t, height));
 	wl_surface_set_opaque_region(window->surface, window->opaque_region);
 	return UWAC_SUCCESS;
 }
@@ -576,102 +684,145 @@ UwacReturnCode UwacWindowSetInputRegion(UwacWindow* window, uint32_t x, uint32_t
 	if (!window->input_region)
 		return UWAC_ERROR_NOMEMORY;
 
-	wl_region_add(window->input_region, x, y, width, height);
+	wl_region_add(window->input_region, WINPR_ASSERTING_INT_CAST(int32_t, x),
+	              WINPR_ASSERTING_INT_CAST(int32_t, y), WINPR_ASSERTING_INT_CAST(int32_t, width),
+	              WINPR_ASSERTING_INT_CAST(int32_t, height));
 	wl_surface_set_input_region(window->surface, window->input_region);
 	return UWAC_SUCCESS;
 }
 
-
 void* UwacWindowGetDrawingBuffer(UwacWindow* window)
 {
-	return window->drawingBuffer->data;
+	UwacBuffer* buffer = NULL;
+
+	if (window->drawingBufferIdx < 0)
+		return NULL;
+
+	buffer = &window->buffers[window->drawingBufferIdx];
+	if (!buffer)
+		return NULL;
+
+	return buffer->data;
 }
 
 static void frame_done_cb(void* data, struct wl_callback* callback, uint32_t time);
 
-static const struct wl_callback_listener frame_listener =
-{
-	frame_done_cb
-};
+static const struct wl_callback_listener frame_listener = { frame_done_cb };
 
+#ifdef UWAC_HAVE_PIXMAN_REGION
+static void damage_surface(UwacWindow* window, UwacBuffer* buffer, int scale)
+{
+	int nrects = 0;
+	const pixman_box32_t* box = pixman_region32_rectangles(&buffer->damage, &nrects);
+
+	for (int i = 0; i < nrects; i++, box++)
+	{
+		const int x = ((int)floor(box->x1 / scale)) - 1;
+		const int y = ((int)floor(box->y1 / scale)) - 1;
+		const int w = ((int)ceil((box->x2 - box->x1) / scale)) + 2;
+		const int h = ((int)ceil((box->y2 - box->y1) / scale)) + 2;
+		wl_surface_damage(window->surface, x, y, w, h);
+	}
+
+	pixman_region32_clear(&buffer->damage);
+}
+#else
+static void damage_surface(UwacWindow* window, UwacBuffer* buffer, int scale)
+{
+	uint32_t nrects = 0;
+	const RECTANGLE_16* boxes = region16_rects(&buffer->damage, &nrects);
+
+	for (UINT32 i = 0; i < nrects; i++)
+	{
+		const RECTANGLE_16* box = &boxes[i];
+		const double dx = floor(1.0 * box->left / scale);
+		const double dy = floor(1.0 * box->top / scale);
+		const double dw = ceil(1.0 * (box->right - box->left) / scale);
+		const double dh = ceil(1.0 * (box->bottom - box->top) / scale);
+		const int x = ((int)dx) - 1;
+		const int y = ((int)dy) - 1;
+		const int w = ((int)dw) + 2;
+		const int h = ((int)dh) + 2;
+		wl_surface_damage(window->surface, x, y, w, h);
+	}
+
+	region16_clear(&buffer->damage);
+}
+#endif
 
 static void UwacSubmitBufferPtr(UwacWindow* window, UwacBuffer* buffer)
 {
-	UINT32 nrects, i;
-#ifdef HAVE_PIXMAN_REGION
-	const pixman_box32_t* box;
-#else
-	const RECTANGLE_16* box;
-#endif
 	wl_surface_attach(window->surface, buffer->wayland_buffer, 0, 0);
-#if 0
-#ifdef HAVE_PIXMAN_REGION
-	box = pixman_region32_rectangles(&buffer->damage, &nrects);
 
-	for (i = 0; i < nrects; i++, box++)
-		wl_surface_damage(window->surface, box->x1, box->y1, (box->x2 - box->x1), (box->y2 - box->y1));
+	int scale = window->display->actual_scale;
+	damage_surface(window, buffer, scale);
 
-#else
-	box = region16_rects(&buffer->damage, &nrects);
-
-	for (i = 0; i < nrects; i++, box++)
-		wl_surface_damage(window->surface, box->left, box->top, (box->right - box->left),
-		                  (box->bottom - box->top));
-
-#endif
-#else
-	wl_surface_damage(window->surface, 0, 0, window->width, window->height);
-#endif
 	struct wl_callback* frame_callback = wl_surface_frame(window->surface);
 	wl_callback_add_listener(frame_callback, &frame_listener, window);
 	wl_surface_commit(window->surface);
-#ifdef HAVE_PIXMAN_REGION
-	pixman_region32_clear(&buffer->damage);
-#else
-	region16_clear(&buffer->damage);
-#endif
+	buffer->dirty = false;
 }
 
-
 static void frame_done_cb(void* data, struct wl_callback* callback, uint32_t time)
 {
 	UwacWindow* window = (UwacWindow*)data;
-	UwacFrameDoneEvent* event;
+	UwacFrameDoneEvent* event = NULL;
 
 	wl_callback_destroy(callback);
-	window->pendingBuffer = NULL;
+	window->pendingBufferIdx = -1;
 	event = (UwacFrameDoneEvent*)UwacDisplayNewEvent(window->display, UWAC_EVENT_FRAME_DONE);
 
 	if (event)
 		event->window = window;
 }
 
-
+#ifdef UWAC_HAVE_PIXMAN_REGION
 UwacReturnCode UwacWindowAddDamage(UwacWindow* window, uint32_t x, uint32_t y, uint32_t width,
                                    uint32_t height)
 {
-#ifdef HAVE_PIXMAN_REGION
+	UwacBuffer* buf = NULL;
 
-	if (!pixman_region32_union_rect(&window->drawingBuffer->damage, &window->drawingBuffer->damage, x,
-	                                y, width, height))
+	if (window->drawingBufferIdx < 0)
+		return UWAC_ERROR_INTERNAL;
+
+	buf = &window->buffers[window->drawingBufferIdx];
+	if (!pixman_region32_union_rect(&buf->damage, &buf->damage, x, y, width, height))
+		return UWAC_ERROR_INTERNAL;
+
+	buf->dirty = true;
+	return UWAC_SUCCESS;
+}
 #else
+UwacReturnCode UwacWindowAddDamage(UwacWindow* window, uint32_t x, uint32_t y, uint32_t width,
+                                   uint32_t height)
+{
 	RECTANGLE_16 box;
+	UwacBuffer* buf = NULL;
 
 	box.left = x;
 	box.top = y;
 	box.right = x + width;
 	box.bottom = y + height;
 
-	if (!region16_union_rect(&window->drawingBuffer->damage, &window->drawingBuffer->damage, &box))
-#endif
+	if (window->drawingBufferIdx < 0)
 		return UWAC_ERROR_INTERNAL;
 
+	buf = &window->buffers[window->drawingBufferIdx];
+	if (!buf)
+		return UWAC_ERROR_INTERNAL;
+
+	if (!region16_union_rect(&buf->damage, &buf->damage, &box))
+		return UWAC_ERROR_INTERNAL;
+
+	buf->dirty = true;
 	return UWAC_SUCCESS;
 }
+#endif
 
-UwacReturnCode UwacWindowGetDrawingBufferGeometry(UwacWindow* window, UwacSize* geometry, size_t* stride)
+UwacReturnCode UwacWindowGetDrawingBufferGeometry(UwacWindow* window, UwacSize* geometry,
+                                                  size_t* stride)
 {
-	if (!window || !window->drawingBuffer)
+	if (!window || (window->drawingBufferIdx < 0))
 		return UWAC_ERROR_INTERNAL;
 
 	if (geometry)
@@ -688,21 +839,30 @@ UwacReturnCode UwacWindowGetDrawingBufferGeometry(UwacWindow* window, UwacSize*
 
 UwacReturnCode UwacWindowSubmitBuffer(UwacWindow* window, bool copyContentForNextFrame)
 {
-	UwacBuffer* drawingBuffer = window->drawingBuffer;
+	UwacBuffer* currentDrawingBuffer = NULL;
+	UwacBuffer* nextDrawingBuffer = NULL;
+	UwacBuffer* pendingBuffer = NULL;
 
-	if (window->pendingBuffer)
+	if (window->drawingBufferIdx < 0)
+		return UWAC_ERROR_INTERNAL;
+
+	currentDrawingBuffer = &window->buffers[window->drawingBufferIdx];
+
+	if ((window->pendingBufferIdx >= 0) || !currentDrawingBuffer->dirty)
 		return UWAC_SUCCESS;
 
-	window->pendingBuffer = window->drawingBuffer;
-	window->drawingBuffer = UwacWindowFindFreeBuffer(window);
+	window->pendingBufferIdx = window->drawingBufferIdx;
+	nextDrawingBuffer = UwacWindowFindFreeBuffer(window, &window->drawingBufferIdx);
+	pendingBuffer = &window->buffers[window->pendingBufferIdx];
 
-	if (!window->drawingBuffer)
+	if ((!nextDrawingBuffer) || (window->drawingBufferIdx < 0))
 		return UWAC_ERROR_NOMEMORY;
 
 	if (copyContentForNextFrame)
-		memcpy(window->drawingBuffer->data, window->pendingBuffer->data, window->stride * window->height);
+		memcpy(nextDrawingBuffer->data, pendingBuffer->data,
+		       1ull * window->stride * window->height);
 
-	UwacSubmitBufferPtr(window, drawingBuffer);
+	UwacSubmitBufferPtr(window, pendingBuffer);
 	return UWAC_SUCCESS;
 }
 
@@ -715,9 +875,8 @@ UwacReturnCode UwacWindowGetGeometry(UwacWindow* window, UwacSize* geometry)
 	return UWAC_SUCCESS;
 }
 
-
 UwacReturnCode UwacWindowSetFullscreenState(UwacWindow* window, UwacOutput* output,
-        bool isFullscreen)
+                                            bool isFullscreen)
 {
 	if (window->xdg_toplevel)
 	{
@@ -735,8 +894,7 @@ UwacReturnCode UwacWindowSetFullscreenState(UwacWindow* window, UwacOutput* outp
 		if (isFullscreen)
 		{
 			wl_shell_surface_set_fullscreen(window->shell_surface,
-			                                WL_SHELL_SURFACE_FULLSCREEN_METHOD_DEFAULT,
-			                                0,
+			                                WL_SHELL_SURFACE_FULLSCREEN_METHOD_DEFAULT, 0,
 			                                output ? output->output : NULL);
 		}
 		else
@@ -755,3 +913,9 @@ void UwacWindowSetTitle(UwacWindow* window, const char* name)
 	else if (window->shell_surface)
 		wl_shell_surface_set_title(window->shell_surface, name);
 }
+
+void UwacWindowSetAppId(UwacWindow* window, const char* app_id)
+{
+	if (window->xdg_toplevel)
+		xdg_toplevel_set_app_id(window->xdg_toplevel, app_id);
+}
