@@ -47,19 +47,16 @@ extern "C"
 	/** @struct wObject
 	 *  @brief This struct contains function pointer to initialize/free objects
 	 *
-	 *  @var fnObjectNew  A new function that creates a clone of the input
-	 *  @var fnObjectInit A function initializing an object, but not allocating it
-	 *  @var fnObjectUninit A function to deinitialize an object, but not free it
-	 *  @var fnObjectFree A function freeing an object
-	 *  @var fnObjectEquals A function to compare two objects
 	 */
 	typedef struct
 	{
-		OBJECT_NEW_FN fnObjectNew;
-		OBJECT_INIT_FN fnObjectInit;
-		OBJECT_UNINIT_FN fnObjectUninit;
-		OBJECT_FREE_FN fnObjectFree;
-		OBJECT_EQUALS_FN fnObjectEquals;
+		OBJECT_NEW_FN fnObjectNew; /**< A new function that creates a clone of the input */
+		OBJECT_INIT_FN
+		fnObjectInit; /**< A function initializing an object, but not allocating it */
+		OBJECT_UNINIT_FN
+		fnObjectUninit;              /**< A function to deinitialize an object, but not free it */
+		OBJECT_FREE_FN fnObjectFree; /**< A function freeing an object */
+		OBJECT_EQUALS_FN fnObjectEquals; /**< A function to compare two objects */
 	} wObject;
 
 	/* utility function with compatible arguments for string data */
@@ -188,6 +185,7 @@ extern "C"
 	 *  @return A newly allocated queue or \b NULL in case of failure
 	 */
 	WINPR_ATTR_MALLOC(Queue_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wQueue* Queue_New(BOOL synchronized, SSIZE_T capacity, SSIZE_T growthFactor);
 
 	/* System.Collections.Stack */
@@ -210,6 +208,7 @@ extern "C"
 	WINPR_API void Stack_Free(wStack* stack);
 
 	WINPR_ATTR_MALLOC(Stack_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wStack* Stack_New(BOOL synchronized);
 
 	/* System.Collections.ArrayList */
@@ -257,6 +256,7 @@ extern "C"
 	WINPR_API void ArrayList_Free(wArrayList* arrayList);
 
 	WINPR_ATTR_MALLOC(ArrayList_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wArrayList* ArrayList_New(BOOL synchronized);
 
 	/* System.Collections.DictionaryBase */
@@ -402,6 +402,7 @@ extern "C"
 	 *  @return A newly allocated dictionary or \b NULL in case of failure
 	 */
 	WINPR_ATTR_MALLOC(ListDictionary_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wListDictionary* ListDictionary_New(BOOL synchronized);
 
 	/* System.Collections.Generic.LinkedList<T> */
@@ -529,6 +530,7 @@ extern "C"
 	 * @return A pointer to the newly allocated linked list or \b NULL in case of failure
 	 */
 	WINPR_ATTR_MALLOC(LinkedList_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wLinkedList* LinkedList_New(void);
 
 	/** @brief Return the \b wObject function pointers for list elements
@@ -615,6 +617,7 @@ extern "C"
 	 *  @return The newly allocated event or \b NULL in case of failure
 	 */
 	WINPR_ATTR_MALLOC(CountdownEvent_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wCountdownEvent* CountdownEvent_New(size_t initialCount);
 
 	/* Hash Table */
@@ -654,6 +657,7 @@ extern "C"
 	WINPR_API void HashTable_Free(wHashTable* table);
 
 	WINPR_ATTR_MALLOC(HashTable_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wHashTable* HashTable_New(BOOL synchronized);
 
 	WINPR_API void HashTable_Lock(wHashTable* table);
@@ -681,6 +685,7 @@ extern "C"
 	WINPR_API void BufferPool_Free(wBufferPool* pool);
 
 	WINPR_ATTR_MALLOC(BufferPool_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wBufferPool* BufferPool_New(BOOL synchronized, SSIZE_T fixedSize, DWORD alignment);
 
 	/* ObjectPool */
@@ -696,6 +701,7 @@ extern "C"
 	WINPR_API void ObjectPool_Free(wObjectPool* pool);
 
 	WINPR_ATTR_MALLOC(ObjectPool_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wObjectPool* ObjectPool_New(BOOL synchronized);
 
 	/* Message Queue */
@@ -769,6 +775,7 @@ extern "C"
 	 * \return A pointer to a newly allocated MessageQueue or NULL.
 	 */
 	WINPR_ATTR_MALLOC(MessageQueue_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wMessageQueue* MessageQueue_New(const wObject* callback);
 
 	/* Message Pipe */
@@ -784,6 +791,7 @@ extern "C"
 	WINPR_API void MessagePipe_Free(wMessagePipe* pipe);
 
 	WINPR_ATTR_MALLOC(MessagePipe_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wMessagePipe* MessagePipe_New(void);
 
 	/* Publisher/Subscriber Pattern */
@@ -821,20 +829,20 @@ extern "C"
 	typedef void (*p##name##EventHandler)(void* context, const name##EventArgs* e)
 
 #define DEFINE_EVENT_RAISE(name)                                                                \
-	static INLINE int PubSub_On##name(wPubSub* pubSub, void* context, const name##EventArgs* e) \
+	static inline int PubSub_On##name(wPubSub* pubSub, void* context, const name##EventArgs* e) \
 	{                                                                                           \
 		WINPR_ASSERT(e);                                                                        \
 		return PubSub_OnEvent(pubSub, #name, context, &e->e);                                   \
 	}
 
 #define DEFINE_EVENT_SUBSCRIBE(name)                                                              \
-	static INLINE int PubSub_Subscribe##name(wPubSub* pubSub, p##name##EventHandler EventHandler) \
+	static inline int PubSub_Subscribe##name(wPubSub* pubSub, p##name##EventHandler EventHandler) \
 	{                                                                                             \
 		return PubSub_Subscribe(pubSub, #name, EventHandler);                                     \
 	}
 
 #define DEFINE_EVENT_UNSUBSCRIBE(name)                                             \
-	static INLINE int PubSub_Unsubscribe##name(wPubSub* pubSub,                    \
+	static inline int PubSub_Unsubscribe##name(wPubSub* pubSub,                    \
 	                                           p##name##EventHandler EventHandler) \
 	{                                                                              \
 		return PubSub_Unsubscribe(pubSub, #name, EventHandler);                    \
@@ -879,6 +887,7 @@ extern "C"
 	WINPR_API void PubSub_Free(wPubSub* pubSub);
 
 	WINPR_ATTR_MALLOC(PubSub_Free, 1)
+	WINPR_ATTR_NODISCARD
 	WINPR_API wPubSub* PubSub_New(BOOL synchronized);
 
 #ifdef __cplusplus
