@@ -44,34 +44,37 @@ static const SCardApiFunctionTable* g_SCardApi = NULL;
 #define xstr(s) str(s)
 #define str(s) #s
 
-#define SCARDAPI_STUB_CALL_LONG(_name, ...)                                              \
-	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);            \
-	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                          \
-	{                                                                                    \
-		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p", \
-		         g_SCardApi, g_SCardApi ? g_SCardApi->pfn##_name : NULL);                \
-		return SCARD_E_NO_SERVICE;                                                       \
-	}                                                                                    \
+#define SCARDAPI_STUB_CALL_LONG(_name, ...)                                                       \
+	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);                     \
+	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                                   \
+	{                                                                                             \
+		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p",          \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi),                                  \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi ? g_SCardApi->pfn##_name : NULL)); \
+		return SCARD_E_NO_SERVICE;                                                                \
+	}                                                                                             \
 	return g_SCardApi->pfn##_name(__VA_ARGS__)
 
-#define SCARDAPI_STUB_CALL_HANDLE(_name)                                                 \
-	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);            \
-	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                          \
-	{                                                                                    \
-		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p", \
-		         g_SCardApi, g_SCardApi ? g_SCardApi->pfn##_name : NULL);                \
-		return NULL;                                                                     \
-	}                                                                                    \
+#define SCARDAPI_STUB_CALL_HANDLE(_name)                                                          \
+	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);                     \
+	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                                   \
+	{                                                                                             \
+		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p",          \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi),                                  \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi ? g_SCardApi->pfn##_name : NULL)); \
+		return NULL;                                                                              \
+	}                                                                                             \
 	return g_SCardApi->pfn##_name()
 
-#define SCARDAPI_STUB_CALL_VOID(_name)                                                   \
-	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);            \
-	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                          \
-	{                                                                                    \
-		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p", \
-		         g_SCardApi, g_SCardApi ? g_SCardApi->pfn##_name : NULL);                \
-		return;                                                                          \
-	}                                                                                    \
+#define SCARDAPI_STUB_CALL_VOID(_name)                                                            \
+	InitOnceExecuteOnce(&g_Initialized, InitializeSCardApiStubs, NULL, NULL);                     \
+	if (!g_SCardApi || !g_SCardApi->pfn##_name)                                                   \
+	{                                                                                             \
+		WLog_DBG(TAG, "Missing function pointer g_SCardApi=%p->" xstr(pfn##_name) "=%p",          \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi),                                  \
+		         WINPR_CXX_COMPAT_CAST(const void*, g_SCardApi ? g_SCardApi->pfn##_name : NULL)); \
+		return;                                                                                   \
+	}                                                                                             \
 	g_SCardApi->pfn##_name()
 
 /**
