@@ -33,9 +33,10 @@ struct s_wLogLayout
 	LPSTR FormatString;
 };
 
-void WLog_Layout_Free(wLog* log, wLogLayout* layout);
+WINPR_LOCAL void WLog_Layout_Free(wLog* log, wLogLayout* layout);
 
 WINPR_ATTR_MALLOC(WLog_Layout_Free, 2)
-wLogLayout* WLog_Layout_New(wLog* log);
+WINPR_ATTR_NODISCARD
+WINPR_LOCAL wLogLayout* WLog_Layout_New(wLog* log);
 
 #endif /* WINPR_WLOG_LAYOUT_PRIVATE_H */
